package com.emall.flash.api.controller.cms;

import com.emall.flash.bean.entity.system.User;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.JsonUtil;
import com.emall.flash.utils.Lists;
import com.emall.flash.web.controller.BaseController;
import com.emall.flash.bean.core.BussinessLog;

import com.emall.flash.bean.entity.cms.Banner;
import com.emall.flash.bean.enumeration.Permission;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.service.cms.BannerService;
import com.emall.flash.utils.StringUtil;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * banner管理
 */
@RestController
@RequestMapping("/banner")
public class BannerMgrController extends BaseController {
    @Autowired
    private BannerService bannerService;
    @Autowired
    private ManagerService managerService;

    @RequestMapping(method = RequestMethod.POST)
    @BussinessLog(value = "编辑banner", key = "title")
    @RequiresPermissions(value = {Permission.BANNER_EDIT})
    public Object save(@ModelAttribute @Valid Banner banner) {
        if(StringUtil.isNotEmpty(banner.getParam())){
            if(!JsonUtil.isJson(banner.getParam())){
                return Rets.failure("参数必须为json格式");
            }
        }
        if(banner.getId()==null){
            User loginUser = managerService.get(JwtUtil.getUserId());
            banner.setTenantId(loginUser.getTenantId());
            bannerService.insert(banner);
        }else {
            bannerService.update(banner);
        }
        return Rets.success();
    }

    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除banner", key = "id")
    @RequiresPermissions(value = {Permission.BANNER_DEL})
    public Object remove(Long id) {
        bannerService.deleteById(id);
        return Rets.success();
    }

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    @RequiresPermissions(value = {Permission.BANNER})
    public Object list(@RequestParam(required = false) String title,@RequestParam(required = false) Long tenantId) {
        List<SearchFilter> filters = Lists.newArrayList();
        if(StringUtil.isNotEmpty(title)){
            filters.add(SearchFilter.build("title", SearchFilter.Operator.LIKE,title));
        }
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null)
            filters.add(SearchFilter.build("tenantId",loginUser.getTenantId()));
        else if (tenantId!=null)
            filters.add(SearchFilter.build("tenantId",tenantId));
        filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
        List<Banner> list = bannerService.queryAll(filters);
        return Rets.success(list);
    }
}
