package com.emall.flash.api.controller.cms;

import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.Lists;
import com.emall.flash.web.controller.BaseController;
import com.emall.flash.bean.core.BussinessLog;

import com.emall.flash.bean.entity.cms.Channel;
import com.emall.flash.bean.enumeration.Permission;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.service.cms.ChannelService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 文章类别管理
 */
@RestController
@RequestMapping("/channel")
public class ChannelMgrController extends BaseController {
    @Autowired
    private ChannelService channelService;
    @Autowired
    private ManagerService managerService;

    @RequestMapping(method = RequestMethod.POST)
    @BussinessLog(value = "编辑栏目", key = "name")
    @RequiresPermissions(value = {Permission.CHANNEL_EDIT})
    public Object save(@ModelAttribute @Valid Channel channel) {
        if(channel.getId()==null) {
            User loginUser = managerService.get(JwtUtil.getUserId());
            channel.setTenantId(loginUser.getTenantId());
            channelService.insert(channel);
        }else{
            channelService.update(channel);
        }
        return Rets.success();
    }

    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除栏目", key = "id")
    @RequiresPermissions(value = {Permission.CHANNEL_DEL})
    public Object remove(Long id) {
        Channel channel =  channelService.get(id);
        channel.setDelFlag(1);
        channelService.update(channel);
        return Rets.success();
    }

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    @RequiresPermissions(value = {Permission.CHANNEL})
    public Object list(@RequestParam(value = "tenantId", required = false) Long tenantId) {
        List<SearchFilter> filters = Lists.newArrayList();
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null)
            filters.add(SearchFilter.build("tenantId",loginUser.getTenantId()));
        else if (tenantId!=null)
            filters.add(SearchFilter.build("tenantId",tenantId));
        filters.add(SearchFilter.build("delFlag",0));
        List<Channel> list = channelService.queryAll(filters);
        return Rets.success(list);
    }
}
