package com.emall.flash.api.controller.cms;

import com.emall.flash.bean.constant.CfgKey;
import com.emall.flash.bean.entity.shop.Tenant;
import com.emall.flash.service.shop.TenantService;
import com.emall.flash.service.system.CfgService;
import com.emall.flash.service.system.impl.ConstantFactory;
import com.emall.flash.utils.Lists;
import com.emall.flash.web.controller.BaseController;
import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.entity.system.FileInfo;
import com.emall.flash.bean.enumeration.Permission;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.service.system.FileService;
import com.emall.flash.utils.StringUtil;
import com.emall.flash.utils.factory.Page;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

@RestController
@RequestMapping("/fileMgr")
public class FileMgrController extends BaseController {

    @Autowired
    private FileService fileService;
    @Autowired
    private TenantService tenantService;
    @Autowired
    private CfgService cfgService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    @RequiresPermissions(value = {Permission.FILE})
    public Object list(@RequestParam(required = false) String originalFileName,@RequestParam(required = false) String tenantId) {
        Page<FileInfo> page = new PageFactory<FileInfo>().defaultPage();
        if (StringUtil.isNotEmpty(originalFileName)) {
            page.addFilter(SearchFilter.build("originalFileName", SearchFilter.Operator.LIKE, originalFileName));
        }
        if (StringUtil.isNotEmpty(tenantId)) {
            page.addFilter(SearchFilter.build("tenantId", SearchFilter.Operator.EQ, tenantId));
        }
        page.addFilter(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
        page = fileService.queryPage(page);
        page.getRecords().forEach(f -> {
            f.setCreateName(ConstantFactory.me().getUserNameById(f.getCreateBy()));
        });
        return Rets.success(page);
    }

    //查询商圈文件数
    @RequestMapping(value = "/getShopTenantFile", method = RequestMethod.GET)
    public Object getShopTenantFile() {
        List<Map> list = tenantService.queryBySql("select t.id,t.name,count(f.id) sum from t_shop_tenant t " +
                "LEFT JOIN t_sys_file_info f ON f.tenant_id = t.id where t.del_flag = 0 GROUP BY t.id order by t.sort ");
        Map map = new HashMap<>();
        map.put("id", "default");
        map.put("name", "平台");
        List<SearchFilter> filters = Lists.newArrayList();
        filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
        filters.add(SearchFilter.build("tenantId", SearchFilter.Operator.EQ,"default"));
        long sum = fileService.count(filters);
        map.put("sum", sum);
        list.add(map);
        return Rets.success(list);
    }


    /**
     * 打包下载商圈文件
     * @param tenantId
     */
    @RequestMapping(value="download",method = RequestMethod.GET)
    public void download(@RequestParam(value = "tenantId",required = false) String tenantId, HttpServletResponse response){
        ZipOutputStream out = null;
        BufferedOutputStream bos = null;
        try {
            //将zip以流的形式输出到前台
//            response.setHeader("content-type", "application/octet-stream");
//            response.setContentType("application/x-download");
            response.setContentType("multipart/form-data");
            response.setCharacterEncoding("utf-8");
            // 设置浏览器响应头对应的Content-disposition
            //参数中 testZip 为压缩包文件名，尾部的.zip 为文件后缀
            response.setHeader("Content-disposition",
                    "attachment;filename=" + new String("ocn".getBytes("gbk"), "iso8859-1")+".zip");
            //创建zip输出流
            out = new ZipOutputStream(response.getOutputStream());
            //创建缓冲输出流
            bos = new BufferedOutputStream(out);
            File sourceFile = new File(cfgService.getCfgValue(CfgKey.SYSTEM_FILE_UPLOAD_PATH) + File.separator + tenantId);
            //调用压缩函数
            compress(out, bos, sourceFile, sourceFile.getName());
            out.flush();
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
        }finally {
            try {
                if (null != bos)
                    bos.close();
            } catch (IOException e) {
                logger.error(e.getMessage(),e);
            }
            try {
                if (null != out)
                    out.close();
            } catch (IOException e) {
                logger.error(e.getMessage(),e);
            }
        }
    }

    /**
     * 文件压缩
     * @param out
     * @param bos
     * @param sourceFile
     * @param base
     */
    public static void compress(ZipOutputStream out, BufferedOutputStream bos, File sourceFile, String base){
        FileInputStream fos = null;
        BufferedInputStream bis = null;
        try {
            //如果路径为目录（文件夹）
            if (sourceFile.isDirectory()) {
                //取出文件夹中的文件（或子文件夹）
                File[] flist = sourceFile.listFiles();
                if (flist.length == 0) {//如果文件夹为空，则只需在目的地zip文件中写入一个目录进入点
                    out.putNextEntry(new ZipEntry(base + "/"));
                } else {//如果文件夹不为空，则递归调用compress，文件夹中的每一个文件（或文件夹）进行压缩
                    for (int i = 0; i < flist.length; i++) {
                        compress(out, bos, flist[i], base + "/" + flist[i].getName());
                    }
                }
            } else {//如果不是目录（文件夹），即为文件，则先写入目录进入点，之后将文件写入zip文件中
                out.putNextEntry(new ZipEntry(base));
                fos = new FileInputStream(sourceFile);
                bis = new BufferedInputStream(fos);

                int tag;
                //将源文件写入到zip文件中
                while ((tag = bis.read()) != -1) {
                    out.write(tag);
                }
                bis.close();
                fos.close();
            }
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
        }finally {
            try {
                if (null != bis)
                    bis.close();
            } catch (IOException e) {
                logger.error(e.getMessage(),e);
            }
            try {
                if (null != fos)
                    fos.close();
            } catch (IOException e) {
                logger.error(e.getMessage(),e);
            }
        }
    }


}


