package com.emall.flash.api.controller.promotion;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.promotion.Topic;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.enumeration.Permission;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.promotion.TopicService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.DateUtil;
import com.emall.flash.utils.StringUtil;
import com.emall.flash.utils.factory.Page;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/promotion/topic")
public class TopicController {
	private  Logger logger = LoggerFactory.getLogger(getClass());
	@Autowired
	private TopicService topicService;
	@Autowired
	private ManagerService managerService;

	@RequestMapping(value = "/list",method = RequestMethod.GET)
	public Object list(@RequestParam(value = "disabled",required = false) Boolean disabled,
					   @RequestParam(value = "startDate",required = false) String startDate,
					   @RequestParam(value = "endDate",required = false) String endDate,
					   @RequestParam(value = "tenantId", required = false) Long tenantId) {
		Page<Topic> page = new PageFactory<Topic>().defaultPage();
		page.addFilter( "disabled", disabled);
		User loginUser = managerService.get(JwtUtil.getUserId());
		if (loginUser.getTenantId()!=null)
			page.addFilter(SearchFilter.build("tenantId",loginUser.getTenantId()));
		else if (tenantId!=null)
			page.addFilter(SearchFilter.build("tenantId",tenantId));
		page.addFilter(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
		page.addFilter("createTime", SearchFilter.Operator.GTE, DateUtil.parse(startDate,"yyyyMMddHHmmss"));
		page.addFilter("createTime", SearchFilter.Operator.LTE, DateUtil.parse(endDate,"yyyyMMddHHmmss"));
		page = topicService.queryPage(page);
		return Rets.success(page);
	}

	@RequestMapping(method = RequestMethod.POST)
	@BussinessLog(value = "编辑专题", key = "name")
	@RequiresPermissions(value = {Permission.TOPIC_EDIT})
	public Object save(@ModelAttribute Topic topic){
		if(topic.getId()==null){
			topic.setPv(0L);
			User loginUser = managerService.get(JwtUtil.getUserId());
			topic.setTenantId(loginUser.getTenantId());
			topicService.insert(topic);
		}else {
			topicService.update(topic);
		}
		return Rets.success();
	}
	@RequestMapping(method = RequestMethod.DELETE)
	@BussinessLog(value = "删除专题", key = "id")
	@RequiresPermissions(value = {Permission.TOPIC_DEL})
	public Object remove(Long id){
		if (StringUtil.isEmpty(id)) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		Topic topic = topicService.get(id);
		topic.setDelFlag(1);
		topicService.update(topic);
		return Rets.success();
	}

	@RequestMapping(value="/changeDisabled",method = RequestMethod.POST)
	@RequiresPermissions(value = {Permission.TOPIC_EDIT})
	public Object changeIsOnSale(@RequestParam("id")  Long id,@RequestParam("disabled") Boolean disabled){
		if (id == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		topicService.changeDisabled(id,disabled);
		return Rets.success();
	}
}
