package com.emall.flash.api.controller.shop;

import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.AttrKey;
import com.emall.flash.bean.entity.shop.AttrVal;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.service.shop.AttrKeyService;
import com.emall.flash.service.shop.AttrValService;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.Maps;
import com.emall.flash.utils.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/shop/attr/val")
public class AttrValController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private AttrValService attrValService;
    @Autowired
    private AttrKeyService attrKeyService;

    @RequestMapping(value = "/getAttrByCategoryAndGoods/{idCategory}",method = RequestMethod.GET)
    public Object getAttrByCategoryAndGoods(@PathVariable("idCategory") Long idCategory) {
        List<AttrKey> keyList = attrKeyService.queryBy(idCategory);
        List<Long> idAttrKeyList = Lists.newArrayList();
        for(AttrKey attrKey:keyList){
            idAttrKeyList.add(attrKey.getId());
        }
        List<AttrVal> valList = Lists.newArrayList();
        if(!idAttrKeyList.isEmpty()) {
            List<SearchFilter> filters = Lists.newArrayList();
            filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
            filters.add(SearchFilter.build("idAttrKey", SearchFilter.Operator.IN, idAttrKeyList));
            valList = attrValService.queryAll(filters);
        }
        return Rets.success(Maps.newHashMap(
                "keyList", keyList,
                "valList", valList
        ));
    }

    @RequestMapping(value = "getAttrVals", method = RequestMethod.GET)
    public Object getAttrVals(@RequestParam("idAttrKey")Long idAttrKey) {
        List<SearchFilter> filters = Lists.newArrayList();
        filters.add(SearchFilter.build("idAttrKey",idAttrKey));
        filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
        List<AttrVal> list  = attrValService.queryAll(filters);
        return Rets.success(list);
    }
    @RequestMapping(method = RequestMethod.POST)
    @BussinessLog(value = "编辑商品属性值", key = "attrVal")
    public Object save(@RequestParam("idAttrKey")Long idAttrKey,
                       @RequestParam(value = "id",required = false)Long id,
                       @RequestParam("attrVal")String attrVal) {
        AttrVal entity = new AttrVal();
        if(id!=null){
            entity = attrValService.get(id);
        }
        entity = attrValService.get(Lists.newArrayList(
                SearchFilter.build("idAttrKey",idAttrKey),
                SearchFilter.build("attrVal",attrVal),
                SearchFilter.build("delFlag", 0)
        ));
        if (id == null) {
            if(entity==null) {
                entity = new AttrVal();
                entity.setIdAttrKey(idAttrKey);
                entity.setAttrVal(attrVal);
                attrValService.insert(entity);
            }else{
                return  Rets.failure("不能添加重复的规格");
            }
        } else {
            if(entity!=null){
                return  Rets.failure("不能添加重复的规格");
            }else {
                entity = new AttrVal();
                entity.setIdAttrKey(idAttrKey);
                entity.setAttrVal(attrVal);
                attrValService.update(entity);
            }
        }
        return Rets.success();
    }

    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除商品属性值", key = "id")
    public Object remove(Long id) {
        if (StringUtil.isEmpty(id)) {
            throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
        }
        AttrVal attrValEntity = attrValService.get(id);
        attrValEntity.setDelFlag(1);
        attrValService.update(attrValEntity);
        return Rets.success();
    }
    @RequestMapping(value="updateAttrVal",method = RequestMethod.POST)
    @BussinessLog(value = "修改商品属性值", key = "id")
    public Object updateAttrName(@RequestParam("id") Long id,@RequestParam("attrVal") String attrVal){
        if (StringUtil.isEmpty(id)) {
            throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
        }
        AttrVal attrValEntity = attrValService.get(id);
        attrValEntity.setAttrVal(attrVal);
        attrValService.update(attrValEntity);
        return Rets.success();
    }
}
