package com.emall.flash.api.controller.shop;

import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.cms.Banner;
import com.emall.flash.bean.entity.shop.AttrKey;
import com.emall.flash.bean.entity.shop.Category;
import com.emall.flash.bean.entity.shop.CategoryBannerRel;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.enumeration.Permission;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.node.CategoryNode;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.AttrKeyService;
import com.emall.flash.service.shop.CategoryBannerRelService;
import com.emall.flash.service.shop.CategoryService;
import com.emall.flash.service.shop.GoodsService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.Lists;
import com.emall.flash.web.controller.BaseController;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/shop/category")
public class CategoryController extends BaseController {
	private  Logger logger = LoggerFactory.getLogger(getClass());
	@Autowired
	private CategoryService categoryService;
	@Autowired
	private CategoryBannerRelService categoryBannerRelService;
	@Autowired
	private AttrKeyService attrKeyService;
	@Autowired
	private GoodsService goodsService;
	@Autowired
	private ManagerService managerService;

	@RequestMapping(value = "/list",method = RequestMethod.GET)
	public Object list(@RequestParam(value = "tenantId",required = false) Long tenantId) {
		if (tenantId==null) {
			User loginUser = managerService.get(JwtUtil.getUserId());
			tenantId = loginUser.getTenantId();
		}
		List<CategoryNode> list = categoryService.getCategories(tenantId, false);
		return Rets.success(list);
	}
	@RequestMapping(value = "/getAll",method = RequestMethod.GET)
	public Object getAll() {
		User loginUser = managerService.get(JwtUtil.getUserId());
		Long tenantId = loginUser.getTenantId();
		List<SearchFilter> filters = Lists.newArrayList();
		filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
		if (tenantId!=null)
			filters.add(SearchFilter.build("tenantId", SearchFilter.Operator.EQ,tenantId));
		List<Category> categories = categoryService.queryAll(filters);

		return Rets.success(categories);
	}
	@RequestMapping(method = RequestMethod.POST)
	@BussinessLog(value = "编辑商品类别", key = "name")
	@RequiresPermissions(value = {Permission.CATEGORY_EDIT})
	public Object save(@ModelAttribute Category category){
		if(category.getId()==null){
			User loginUser = managerService.get(JwtUtil.getUserId());
			category.setTenantId(loginUser.getTenantId());
			categoryService.insert(category);
		}else {
			Category old = categoryService.get(category.getId());
			category.setCreateBy(old.getCreateBy());
			category.setCreateTime(old.getCreateTime());
			categoryService.update(category);
		}
		return Rets.success();
	}

	@RequestMapping(method = RequestMethod.DELETE)
	@BussinessLog(value = "删除商品类别", key = "id")
	@RequiresPermissions(value = {Permission.CATEGORY_EDIT})
	public Object remove(Long id){
		if (id == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		List<SearchFilter> filters = Lists.newArrayList();
		filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
		filters.add(SearchFilter.build("idCategory",id));
		long goodsCount = goodsService.count(filters);
		if(goodsCount>0){
			throw new ApplicationException(ApplicationExceptionEnum.DATA_CANNOT_REMOVE);
		}
		Category category = categoryService.get(id);
		category.setDelFlag(1);
		categoryService.update(category);
		return Rets.success();
	}
	@RequestMapping(value="/getBanners/{idCategory}",method = RequestMethod.GET)
	public Object getBanners(@PathVariable("idCategory") Long idCategory){
		if (idCategory == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		List<SearchFilter> filters = Lists.newArrayList();
		filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
		filters.add(SearchFilter.build("idCategory", SearchFilter.Operator.EQ,idCategory));
		List<CategoryBannerRel> relList = categoryBannerRelService.queryAll(filters);
		List<Banner> bannerList = Lists.newArrayList();
		relList.forEach( item->{
			bannerList.add(item.getBanner());
		});

		return Rets.success(bannerList);
	}
	@RequestMapping(value ="getAttrKeys/{idCategory}",method = RequestMethod.GET)
	public Object getAttrKeys(@PathVariable("idCategory") Long idCategory){
		if (idCategory == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		List<AttrKey> list = attrKeyService.queryBy(idCategory);
		return Rets.success(list);

	}
	@RequestMapping(value="/removeBanner/{idCategory}/{idBanner}",method = RequestMethod.DELETE)
	@RequiresPermissions(value = {Permission.CATEGORY_EDIT})
	public Object removeBanner(@PathVariable("idCategory") Long idCategory,
							@PathVariable("idBanner") Long idBanner){
		if (idCategory == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		CategoryBannerRel rel = categoryBannerRelService.get(Lists.newArrayList(
				SearchFilter.build("idCategory",idCategory),
				SearchFilter.build("idBanner",idBanner)
		));
		if(rel!=null){
			categoryBannerRelService.delete(rel);
		}
		return Rets.success();
	}
	@RequestMapping(value="/setBanner/{idCategory}/{idBanner}",method = RequestMethod.POST)
	@RequiresPermissions(value = {Permission.CATEGORY_EDIT})
	public Object setBanner(@PathVariable("idCategory") Long idCategory,
							@PathVariable("idBanner") Long idBanner){
		if (idCategory == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		CategoryBannerRel rel = categoryBannerRelService.get(Lists.newArrayList(
				SearchFilter.build("idCategory",idCategory),
				SearchFilter.build("idBanner",idBanner)
		));
		if(rel!=null){
			return Rets.success();
		}
		rel = new CategoryBannerRel();
		rel.setIdCategory(idCategory);
		rel.setIdBanner(idBanner);
		categoryBannerRelService.insert(rel);
		return Rets.success();
	}
	@PostMapping(value="/changeShowIndex/{idCategory}/{showIndex}")
	@RequiresPermissions(value = {Permission.CATEGORY_EDIT})
	public Object changeShowIndex(@PathVariable("idCategory") Long idCategory,
							@PathVariable("showIndex") Boolean showIndex){
		if (idCategory == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		Category category = categoryService.get(idCategory);
		category.setShowIndex(showIndex);
		categoryService.update(category);
		return Rets.success();
	}
}
