package com.emall.flash.api.controller.shop;

import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.cms.Channel;
import com.emall.flash.bean.entity.shop.Category;
import com.emall.flash.bean.entity.shop.CategoryBannerRel;
import com.emall.flash.bean.entity.shop.Columns;
import com.emall.flash.bean.entity.shop.ColumnsGoods;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.enumeration.Permission;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.ModelVo;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.node.CategoryNode;
import com.emall.flash.bean.vo.node.ColumnsNode;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.bean.vo.shop.ColumnsGoodsVo;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.cms.ChannelService;
import com.emall.flash.service.shop.ColumnsGoodsService;
import com.emall.flash.service.shop.ColumnsService;
import com.emall.flash.service.shop.GoodsService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.Lists;
import com.emall.flash.web.controller.BaseController;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * 商品栏目管理
 */
@RestController
@RequestMapping("/columns")
public class ColumnsController extends BaseController {
    @Autowired
    private ColumnsService columnsService;
    @Autowired
    private ColumnsGoodsService columnsGoodsService;
    @Autowired
    private ManagerService managerService;
    @Autowired
    private GoodsService goodsService;

    @RequestMapping(method = RequestMethod.POST)
    @BussinessLog(value = "编辑栏目", key = "name")
    public Object save(@ModelAttribute @Valid Columns columns) {
        if(columns.getId()==null) {
            User loginUser = managerService.get(JwtUtil.getUserId());
            columns.setTenantId(loginUser.getTenantId());
            columnsService.insert(columns);
        }else{
            columnsService.update(columns);
        }
        return Rets.success();
    }

    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除栏目", key = "id")
    public Object remove(Long id) {
//        List<SearchFilter> filters = Lists.newArrayList();
//        filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
//        filters.add(SearchFilter.build("columnsId",id));
//        long goodsCount = goodsService.count(filters);
//        if(goodsCount>0){
//            throw new ApplicationException(ApplicationExceptionEnum.DATA_CANNOT_REMOVE);
//        }
        List<SearchFilter> filter = Lists.newArrayList();
        filter.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
        filter.add(SearchFilter.build("pid",id));
        long count = columnsService.count(filter);
        if(count>0){
            throw new ApplicationException(ApplicationExceptionEnum.DATA_CANNOT_REMOVE);
        }
        Columns columns =  columnsService.get(id);
        columns.setDelFlag(1);
        columnsService.update(columns);
        List<ColumnsGoods> rels = columnsGoodsService.queryAll(SearchFilter.build("columnsId", id));
        columnsGoodsService.deleteAll(rels);
        return Rets.success();
    }

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Object list(@RequestParam(value = "tenantId", required = false) Long tenantId) {
        Sort sort = Sort.by(Sort.Direction.DESC, "createTime");
        List<ColumnsNode> nodes = new ArrayList<>();
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null)
            nodes = columnsService.queryAllNode(loginUser.getTenantId(),sort);
        else
            nodes = columnsService.queryAllNode(tenantId,sort);
        return Rets.success(nodes);
    }

    @RequestMapping(value = "/selectColumnsGoods", method = RequestMethod.GET)
    public Object selectColumnsGoods(@RequestParam(value = "columnsId", required = false) Long columnsId) {
        List<SearchFilter> filters = Lists.newArrayList();
        filters.add(SearchFilter.build("columnsId",columnsId));
        List<ColumnsGoods> list = columnsGoodsService.queryAll(filters);
        return Rets.success(list);
    }


    @RequestMapping(value = "/setColumnsGoods",method = RequestMethod.POST)
    @BussinessLog(value = "编辑栏目商品关联", key = "id")
    public Object setColumnsGoods(@RequestBody ColumnsGoodsVo columnsGoodsVo) {
        List<SearchFilter> filters = Lists.newArrayList();
        filters.add(SearchFilter.build("columnsId",columnsGoodsVo.getColumnsId()));
        List<ColumnsGoods> list = columnsGoodsService.queryAll(filters);
        columnsGoodsService.deleteAll(list);
        for (Long goodsId : columnsGoodsVo.getGoodsIds()) {
            ColumnsGoods goods = new ColumnsGoods();
            goods.setGoodsId(goodsId);
            goods.setColumnsId(columnsGoodsVo.getColumnsId());
            columnsGoodsService.insert(goods);
        }
        return Rets.success();
    }
}
