package com.emall.flash.api.controller.shop;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.*;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.enumeration.Permission;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.*;
import com.emall.flash.service.system.LogObjectHolder;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.factory.Page;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping("/shop/goods")
public class GoodsController {
	private  Logger logger = LoggerFactory.getLogger(getClass());
	@Autowired
	private GoodsService goodsService;
	@Autowired
	private GoodsSkuService goodsSkuService;
	@Autowired
	private ManagerService managerService;
	@Autowired
	private TenantService tenantService;
	@Autowired
	private FavoriteService favoriteService;
	@Autowired
	private ColumnsGoodsService columnsGoodsService;

	@RequestMapping(value = "/list",method = RequestMethod.GET)
	public Object list(@RequestParam(value = "name",required = false) String name,
					   @RequestParam(value = "isOnSale",required = false) Boolean isOnSale,
					   @RequestParam(required = false) List<Long> tenantIds) {
		Page<Goods> page = new PageFactory<Goods>().defaultPage();
		User loginUser = managerService.get(JwtUtil.getUserId());
		if (loginUser.getTenantId()!=null)
			page.addFilter("tenantId", SearchFilter.Operator.EQ,loginUser.getTenantId());
		else if (tenantIds!=null && !tenantIds.isEmpty())
			page.addFilter("tenantId", SearchFilter.Operator.IN,tenantIds);
		if (isOnSale != null)
			page.addFilter("isOnSale", SearchFilter.Operator.EQ,isOnSale);
		if (StringUtils.isNotBlank(name))
			page.addFilter("name", SearchFilter.Operator.LIKE,name);
		page.addFilter(SearchFilter.build("delFlag", 0));
		page.addFilter(SearchFilter.build("isDelete", false));
		page = goodsService.queryPage(page);
		for (Goods record : page.getRecords()) {
			if (record.getTenantId()!=null) {
				Tenant tenant = tenantService.get(record.getTenantId());
				record.setTenantName(tenant.getName());
			}
			List<SearchFilter> filters = Lists.newArrayList();
			filters.add(SearchFilter.build("idGoods",record.getId()));
			filters.add(SearchFilter.build("delFlag",0));
			List<Favorite> list = favoriteService.queryAll(filters);
			record.setSum(list.size());
		}
		return Rets.success(page);
	}

	@RequestMapping(value = "/saveBaseInfo",method = RequestMethod.POST)
	@BussinessLog(value = "保存商品基本信息", key = "name")
	@RequiresPermissions(value = {Permission.GOODS_EDIT})
	public Object saveBaseInfo(@RequestBody Goods goods){
		if(goods.getId()==null){
			User loginUser = managerService.get(JwtUtil.getUserId());
			goods.setTenantId(loginUser.getTenantId());
			goodsService.insert(goods);
		}
		return Rets.success(goods.getId());
	}
	@RequestMapping(method = RequestMethod.POST)
	@BussinessLog(value = "编辑商品", key = "name")
	@RequiresPermissions(value = {Permission.GOODS_EDIT})
	public Object save(@RequestBody @Valid Goods goods){
		List<SearchFilter> filters = Lists.newArrayList(
				SearchFilter.build("idGoods",goods.getId()),
				SearchFilter.build("isDeleted",false)
		);
		List<GoodsSku> skuList = goodsSkuService.queryAll(filters);
		if(goods.getPrice() ==null){
			if(!skuList.isEmpty()){
				int stock = 0;
				for(GoodsSku sku:skuList){
					stock+=sku.getStock();
				}
				goods.setStock(stock);
				goods.setPrice(skuList.get(0).getPrice());
			}else{
				throw new ApplicationException(ApplicationExceptionEnum.DATA_ERROR);
			}
		}else{
			if(!skuList.isEmpty()){
				//如果配置了price，说明是单规格商品，则将之前配置的sku库存皆设置为0;这里最好不要删除sku记录，避免之前下过的订单无法正确关联
				for(GoodsSku sku:skuList){
					sku.setStock(0);
				}
				goodsSkuService.update(skuList);
			}
		}
		if(goods.getId()==null){
			User loginUser = managerService.get(JwtUtil.getUserId());
			goods.setTenantId(loginUser.getTenantId());
			goodsService.insert(goods);
		}else {
			Goods old = goodsService.get(goods.getId());
			LogObjectHolder.me().set(old);
			goods.setCreateBy(old.getCreateBy());
			goods.setCreateTime(old.getCreateTime());
			goodsService.update(goods);
		}
		return Rets.success();
	}
	@RequestMapping(method = RequestMethod.DELETE)
	@BussinessLog(value = "删除商品", key = "id")
	@RequiresPermissions(value = {Permission.GOODS_EDIT})
	public Object remove(Long id){
		if (id == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		goodsService.deleteById(id);
		List<ColumnsGoods> rels = columnsGoodsService.queryAll(SearchFilter.build("goodsId", id));
		columnsGoodsService.deleteAll(rels);
		return Rets.success();
	}

	@RequestMapping(method = RequestMethod.GET)
	public Object get(Long id){
		if (id == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		return Rets.success(goodsService.getDetail(id));
	}
	@RequestMapping(value="/changeIsOnSale",method = RequestMethod.POST)
//	@RequiresPermissions(value = {Permission.GOODS_EDIT})
	@BussinessLog(value = "上架/下架商品", key = "id")
	public Object changeIsOnSale(@RequestParam("id")  Long id,@RequestParam("isOnSale") Boolean isOnSale){
		if (id == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		goodsService.changeIsOnSale(id,isOnSale);
		return Rets.success(goodsService.get(id));
	}

}
