package com.emall.flash.api.controller.shop;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.Parking;
import com.emall.flash.bean.entity.shop.Shop;
import com.emall.flash.bean.entity.shop.Tenant;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.ParkingService;
import com.emall.flash.service.shop.TenantService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.factory.Page;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@RestController
@RequestMapping("/shop/parking")
public class ParkingController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private ParkingService parkingService;
    @Autowired
    private ManagerService managerService;


    /**
     * 获取所有停车场
     */
    @RequestMapping(value = "/list",method = RequestMethod.GET)
    public Object list(@RequestParam(required = false) Long tenantId,
                       @RequestParam(required = false) String name) {
        Page page = new PageFactory().defaultPage();
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null)
            page.addFilter(SearchFilter.build("tenantId", loginUser.getTenantId()));
        else if (tenantId!=null)
            page.addFilter(SearchFilter.build("tenantId", tenantId));
        if (StringUtils.isNotBlank(name))
            page.addFilter(SearchFilter.build("name", SearchFilter.Operator.LIKE,name));
        page.addFilter(SearchFilter.build("delFlag", 0));
        page.setSort(Sort.by(Sort.Direction.DESC, "createTime"));
        page = parkingService.queryPage(page);
        return Rets.success(page);
    }

    @RequestMapping(value = "/add",method = RequestMethod.POST)
    @BussinessLog(value = "添加停车场", key = "id")
    public Object add(@RequestBody Parking parking) {
        parking.setDelFlag(0);
        User loginUser = managerService.get(JwtUtil.getUserId());
        parking.setTenantId(loginUser.getTenantId());
        if (parkingService.isRepeat(parking))
            return Rets.failure("停车场名称重复");
        parkingService.insert(parking);
        return Rets.success(parking);
    }

    @RequestMapping(value = "/update",method = RequestMethod.PUT)
    @BussinessLog(value = "修改停车场", key = "id")
    public Object update(@RequestBody Parking parking) {
        if (parkingService.isRepeat(parking))
            return Rets.failure("停车场名称重复");
        parkingService.update(parking);
        return Rets.success();
    }


    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除停车场", key = "id")
    public Object delete(@RequestParam Long id) {
        Parking parking = parkingService.get(id);
        parking.setDelFlag(1);
        parkingService.update(parking);
        return Rets.success();
    }


}
