package com.emall.flash.api.controller.shop;


import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.constant.state.IntegralStyle;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.*;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.api.WeixinService;
import com.emall.flash.service.shop.ReceiptService;
import com.emall.flash.service.shop.ShopUserService;
import com.emall.flash.service.shop.TenantUserService;
import com.emall.flash.service.shop.UserIntegralService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.factory.Page;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.List;


@RestController
@RequestMapping("/shop/receipt")
public class ReceiptController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ReceiptService receiptService;
    @Autowired
    private ManagerService managerService;
    @Autowired
    private UserIntegralService userIntegralService;
    @Autowired
    private WeixinService weixinService;
    @Autowired
    private ShopUserService shopUserService;
    @Autowired
    private TenantUserService tenantUserService;

    /**
     * 获取获取小票记录
     */
    @RequestMapping(value = "/list",method = RequestMethod.GET)
    public Object list(@RequestParam(required = false) Long tenantId,
                        @RequestParam(required = false) Integer status) {
        Page<Receipt> page = new PageFactory().defaultPage();
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null)
            page.addFilter(SearchFilter.build("tenantId", loginUser.getTenantId()));
        else if (tenantId!=null)
            page.addFilter(SearchFilter.build("tenantId", tenantId));
        if (status!=null)
            page.addFilter(SearchFilter.build("status", status));
        page.addFilter(SearchFilter.build("delFlag", 0));
        page.setSort(Sort.by(Sort.Direction.DESC, "createTime"));
        page = receiptService.queryPage(page);
        return Rets.success(page);
    }

    @RequestMapping(value = "/update",method = RequestMethod.PUT)
    @BussinessLog(value = "审核小票", key = "id")
    @Transactional
    public Object update(@RequestBody Receipt receipt) {
        Receipt rec = receiptService.get(receipt.getId());
        rec.setStatus(receipt.getStatus());
        rec.setIntegral(receipt.getIntegral());
        rec.setRemarks(receipt.getRemarks());
        receiptService.update(rec);
        if (receipt.getStatus().equals(1)) {
            //添加我的积分
            UserIntegral userIntegral = new UserIntegral();
            userIntegral.setIntegral(rec.getIntegral());
            userIntegral.setUserId(rec.getUserId());
            userIntegral.setTenantId(rec.getTenantId());
            userIntegral.setType(0);
            userIntegral.setStyle(IntegralStyle.RECEIPT.getCode());
            userIntegral.setRemarks("上传小票");
            userIntegralService.insert(userIntegral);
            List<SearchFilter> filters = Lists.newArrayList();
            filters.add( SearchFilter.build("userId",rec.getUserId()));
            filters.add( SearchFilter.build("tenantId",rec.getTenantId()));
            filters.add( SearchFilter.build("delFlag",0));
            TenantUser user = tenantUserService.get(filters);
            if (user!=null) {
                user.setIntegral(user.getIntegral() + receipt.getIntegral());
                tenantUserService.update(user);
            } else {
                user = new TenantUser();
                user.setTenantId(rec.getTenantId());
                user.setUserId(rec.getUserId());
                user.setLevel(0);
                user.setIntegral(receipt.getIntegral());
                tenantUserService.insert(user);
            }
        }
        //发送结果通知
        ShopUser user = shopUserService.get(rec.getUserId());
        weixinService.sendSubscribeMessage(receipt,user.getWechatOpenId());
        return Rets.success();
    }

}
