package com.emall.flash.api.controller.shop;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.Shop;
import com.emall.flash.bean.entity.shop.Tenant;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.ShopService;
import com.emall.flash.service.shop.TenantService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.factory.Page;
import org.apache.logging.log4j.util.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@RestController
@RequestMapping("/shop/shop")
public class ShopController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private ShopService shopService;
    @Autowired
    private ManagerService managerService;
    @Autowired
    private TenantService tenantService;

    /**
     * 获取所有商家
     */
    @RequestMapping(value = "/list",method = RequestMethod.GET)
//    @RequiresPermissions(value = {Permission.DICT})
    public Object list( @RequestParam(required = false) String name,@RequestParam(required = false) List<Long> tenantIds) {
        Page<Shop> page = new PageFactory().defaultPage();
        if (Strings.isNotBlank(name))
            page.addFilter( "name",SearchFilter.Operator.LIKE,name);
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null)
            page.addFilter("tenantId", SearchFilter.Operator.EQ,loginUser.getTenantId());
        else if (tenantIds!=null && !tenantIds.isEmpty())
            page.addFilter("tenantId", SearchFilter.Operator.IN,tenantIds);
        page.addFilter( "delFlag",SearchFilter.Operator.EQ,0);
        page.setSort(Sort.by(Sort.Direction.DESC, "createTime"));
        page = shopService.queryPage(page);
        for (Shop record : page.getRecords()) {
            if (record.getTenantId()!=null) {
                Tenant tenant = tenantService.get(record.getTenantId());
                record.setTenantName(tenant.getName());
            }
        }
        return Rets.success(page);
    }

    @RequestMapping(value = "/add",method = RequestMethod.POST)
    @BussinessLog(value = "添加商家", key = "id")
    public Object add(@RequestBody Shop shop) {
        shop.setDelFlag(0);
        if (shopService.isRepeat(shop))
            return Rets.failure("商家名称重复");
        User loginUser = managerService.get(JwtUtil.getUserId());
        shop.setTenantId(loginUser.getTenantId());
        shopService.insert(shop);
        return Rets.success(shop);
    }

    @RequestMapping(value = "/update",method = RequestMethod.PUT)
    @BussinessLog(value = "修改商家", key = "id")
    public Object update(@RequestBody Shop shop) {
        if (shopService.isRepeat(shop))
            return Rets.failure("商家名称重复");
        shopService.update(shop);
        return Rets.success();
    }

    @RequestMapping(value = "/updateStatus",method = RequestMethod.PUT)
    @BussinessLog(value = "停启用商家", key = "id")
    public Object updateStatus(@RequestParam Long id,@RequestParam Integer status) {
        Shop shop = shopService.get(id);
        shop.setStatus(status);
        shopService.update(shop);
        return Rets.success();
    }

    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除商家", key = "id")
    public Object delete(@RequestParam Long id) {
        Shop shop = shopService.get(id);
        shop.setDelFlag(1);
        shopService.update(shop);
        return Rets.success();
    }

    @RequestMapping(value = "/selectShopList", method = RequestMethod.GET)
    public Object selectShopList() {
        List<Shop> list = shopService.queryAll(Lists.newArrayList(
                SearchFilter.build("status", 1),
                SearchFilter.build("delFlag", 0)
        ));
        return Rets.success(list);
    }



}
