package com.emall.flash.api.controller.shop;

import com.emall.flash.api.utils.PhoneUtils;
import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.entity.shop.Order;
import com.emall.flash.bean.entity.shop.ShopUser;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.service.shop.CartService;
import com.emall.flash.service.shop.OrderService;
import com.emall.flash.service.shop.ShopUserService;
import com.emall.flash.utils.DateUtil;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.Maps;
import com.emall.flash.utils.StringUtil;
import com.emall.flash.utils.factory.Page;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;


@RestController
@RequestMapping("/shop/user")
public class ShopUserController {
	private  Logger logger = LoggerFactory.getLogger(getClass());
	@Autowired
	private ShopUserService shopUserService;
	@Autowired
	private CartService cartService;
	@Autowired
	private OrderService orderService;

	@RequestMapping(value = "/list",method = RequestMethod.GET)
	public Object list(@RequestParam(required = false) String mobile, @RequestParam(required = false) String nickName,
					   @RequestParam(required = false) String startRegDate,
					   @RequestParam(required = false) String endRegDate,
					   @RequestParam(required = false) String startLastLoginTime,
					   @RequestParam(required = false) String endLastLoginTime) {
		Page<ShopUser> page = new PageFactory<ShopUser>().defaultPage();
		page.addFilter("mobile",mobile);
		page.addFilter("nickName",nickName);
		if(StringUtil.isNotEmpty(startRegDate)) {
			page.addFilter("createTime", SearchFilter.Operator.GTE, DateUtil.parseTime(startRegDate+" 00:00:00"));
		}
		if(StringUtil.isNotEmpty(endRegDate)) {
			page.addFilter("createTime", SearchFilter.Operator.LTE, DateUtil.parseTime(endRegDate+" 23:59:59"));
		}
		if(StringUtil.isNotEmpty(startLastLoginTime)) {
			page.addFilter("lastLoginTime", SearchFilter.Operator.GTE, DateUtil.parseTime(startLastLoginTime+" 00:00:00"));
		}
		if(StringUtil.isNotEmpty(endLastLoginTime)) {
			page.addFilter("lastLoginTime", SearchFilter.Operator.LTE, DateUtil.parseTime(endLastLoginTime+" 23:59:59"));
		}
		page = shopUserService.queryPage(page);
		for (ShopUser record : page.getRecords()) {
			if (StringUtil.isNotEmpty(record.getMobile()))
				record.setMobile(PhoneUtils.phoneDesensitization(record.getMobile()));
		}
		return Rets.success(page);
	}

	@RequestMapping(value="{id}",method = RequestMethod.GET)
	public Object get(@PathVariable("id") Long id){
		if (id == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		return Rets.success(shopUserService.get(id));
	}

	@RequestMapping(value="/info/{id}",method = RequestMethod.GET)
	public Object getInfo(@PathVariable("id") Long id){
		if (id == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		ShopUser shopUser = shopUserService.get(id);

		List<SearchFilter> filters = Lists.newArrayList();
		filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
		filters.add(SearchFilter.build("idUser",id));
		Long  cartCount = cartService.count(filters);
		Long orderCount = orderService.count(filters);
		ShopUser shopUser1 = new ShopUser();
		BeanUtils.copyProperties(shopUser,shopUser1,"password","salt");
		Map<String,Object> data = Maps.newHashMap(
				"cartCount",cartCount,
				"orderCount",orderCount,
				"info",shopUser1
		);
		return Rets.success(data);
	}

	@RequestMapping(value="/getUserOrders",method = RequestMethod.GET)
	public Object getUserOrders(@RequestParam("id") Long id){
		if (id == null) {
			throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
		}
		Page<Order> page = new PageFactory<Order>().defaultPage();
		page.addFilter(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
		page.addFilter(SearchFilter.build("idUser",id));
		page.setSort(Sort.by(Sort.Direction.DESC, "createTime"));
		page = orderService.queryPage(page);
		return Rets.success(page);
	}
}
