package com.emall.flash.api.controller.shop;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.Tenant;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.service.shop.TenantService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.factory.Page;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@RestController
@RequestMapping("/shop/tenant")
public class TenantController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private TenantService tenantService;
    @Autowired
    private ManagerService managerService;
    @Value("${init.shopAdminRoleId}")
    private String shopAdminRoleId;
    /**
     * 获取所有商圈
     */
    @RequestMapping(value = "/list",method = RequestMethod.GET)
//    @RequiresPermissions(value = {Permission.DICT})
    public Object list(@RequestParam(required = false) Long cityId,
                       @RequestParam(required = false) String name) {
        Page page = new PageFactory().defaultPage();
        page.setSort(Sort.by(Sort.Direction.ASC, "sort"));
        page = tenantService.selectPage(page,cityId,name);
        return Rets.success(page);
    }

    @RequestMapping(value = "/add",method = RequestMethod.POST)
    @BussinessLog(value = "添加商圈", key = "id")
//    @RequiresPermissions(value = {Permission.DICT_EDIT})
    public Object add(@RequestBody Tenant tenant) {
        tenant.setDelFlag(0);
        if (Boolean.TRUE.equals(tenantService.isRepeat(tenant)))
            return Rets.failure("商圈名称重复");
        tenantService.insert(tenant);
        return Rets.success(tenant.getId());
    }

    @RequestMapping(value = "/update",method = RequestMethod.PUT)
    @BussinessLog(value = "修改商圈", key = "id")
//    @RequiresPermissions(value = {Permission.DICT_EDIT})
    public Object update(@RequestBody Tenant tenant) {
        if (Boolean.TRUE.equals(tenantService.isRepeat(tenant)))
            return Rets.failure("商圈名称重复");
        tenantService.update(tenant);
        return Rets.success();
    }


    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除商圈", key = "id")
//    @RequiresPermissions(value = {Permission.DICT_DEL})
    public Object delete(@RequestParam Long id) {
        List<User> userList = managerService.queryAll(
                Lists.newArrayList(SearchFilter.build("tenantId", id),
                        SearchFilter.build("status", 1)));
        if (!userList.isEmpty())
            return Rets.failure("该商圈有关联用户，请先处理");
        Tenant tenant = tenantService.get(id);
        tenant.setDelFlag(1);
        tenantService.update(tenant);
        return Rets.success();
    }

    /**
     * 获取可以配置管理员的用户
     */
    @RequestMapping(value = "/getShopRoleUser",method = RequestMethod.GET)
//    @RequiresPermissions(value = {Permission.DICT})
    public Object getShopRoleUser( @RequestParam(required = false) String name) {
        Page page = new PageFactory().defaultPage();
        page = tenantService.getShopRoleUser(page,name);
        return Rets.success(page);
    }

    @RequestMapping(value = "/setShopAdmin",method = RequestMethod.POST)
    @BussinessLog(value = "配置/解绑商圈管理员", key = "id")
//    @RequiresPermissions(value = {Permission.DICT_EDIT})
    public Object setShopAdmin(@RequestBody Tenant tenant) {
        //校验选中用户是否被其他商圈绑定
        Long userId = tenant.getManager();
        User user = managerService.get(userId);
        if (tenant.getBindType().equals(0)) { //绑定
            if (user.getUserType().equals(1) && !tenant.getId().equals(user.getTenantId()) && user.getTenantId()!=null)
                return Rets.failure("该用户已经被其他商圈绑定管理员");
            List<User> userList = managerService.queryAll(
                    Lists.newArrayList(SearchFilter.build("tenantId", tenant.getId()),
                    SearchFilter.build("userType", 1),
                    SearchFilter.build("id",SearchFilter.Operator.NE,userId),
                    SearchFilter.build("status", 1)));
            if (!userList.isEmpty())
                return Rets.failure("该商圈已经绑定管理员，请先解绑");
            if (user.getUserType().equals(1) && tenant.getId().equals(user.getTenantId()))
                return Rets.success(tenant);
            user.setUserType(1);
            user.setTenantId(tenant.getId());
            //赋予用户商圈管理员角色
            user.setRoleid(user.getRoleid()+shopAdminRoleId+",");
            managerService.update(user);
            Tenant t = tenantService.get(tenant.getId());
            t.setManager(userId);
            tenantService.update(t);
        } else { //解绑
            if (!user.getUserType().equals(1) || !user.getTenantId().equals(tenant.getId()))
                return Rets.failure("该用户不能解除绑定该商圈管理员");
            user.setUserType(2);
            user.setRoleid(user.getRoleid().replace(shopAdminRoleId+",",""));
            managerService.update(user);
            Tenant t = tenantService.get(tenant.getId());
            t.setManager(null);
            tenantService.update(t);
        }
        return Rets.success(tenant.getId());
    }

    /**
     * 获取所有商圈list
     */
    @RequestMapping(value = "/getTenantList",method = RequestMethod.GET)
    public Object getTenantList() {
        List<Tenant> list = tenantService.queryAll(
                Lists.newArrayList(SearchFilter.build("delFlag", SearchFilter.Operator.EQ, 0)),
                Sort.by(Sort.Direction.ASC, "sort"));
        return Rets.success(list);
    }
}
