package com.emall.flash.api.controller.system;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.system.Cfg;
import com.emall.flash.bean.entity.system.FileInfo;
import com.emall.flash.bean.enumeration.Permission;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.service.system.CfgService;
import com.emall.flash.service.system.FileService;
import com.emall.flash.service.system.LogObjectHolder;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.Maps;
import com.emall.flash.utils.StringUtil;
import com.emall.flash.utils.factory.Page;
import com.emall.flash.web.controller.BaseController;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.nutz.json.Json;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * CfgController
 *
 * @author enilu
 * @version 2018/11/17 0017
 */
@RestController
@RequestMapping("/cfg")
public class CfgController extends BaseController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private CfgService cfgService;
    @Autowired
    private FileService fileService;

    /**
     * 查询参数列表
     */
    @RequestMapping(value = "/list",method = RequestMethod.GET)
    @RequiresPermissions(value = {Permission.CFG})
    public Object list(@RequestParam(required = false) String cfgName, @RequestParam(required = false) String cfgValue) {
        Page<Cfg> page = new PageFactory<Cfg>().defaultPage();
        if(StringUtil.isNotEmpty(cfgName)){
            page.addFilter(SearchFilter.build("cfgName", SearchFilter.Operator.LIKE, cfgName));
        }
        if(StringUtil.isNotEmpty(cfgValue)){
            page.addFilter(SearchFilter.build("cfgValue", SearchFilter.Operator.LIKE, cfgValue));
        }
        page.addFilter(SearchFilter.build("delFlag", 0));
        page = cfgService.queryPage(page);
        return Rets.success(page);
    }

    /**
     * 分组查询参数
     * @param cfgGroup
     * @return
     */
    @RequestMapping(value = "/getByPrefix",method = RequestMethod.GET)
    @RequiresPermissions(value = {Permission.CFG})
    public Object list(@RequestParam(required = false) String cfgGroup) {
        List<SearchFilter> filters = Lists.newArrayList();
        filters.add(SearchFilter.build("delFlag", 0));
        filters.add(SearchFilter.build("cfgName", SearchFilter.Operator.LIKEL,cfgGroup));
        List<Cfg> list = cfgService.queryAll(filters);
        Map map = Maps.newHashMap();
        for(Cfg cfg:list){
            map.put(cfg.getCfgName(),cfg.getCfgValue());
        }
        return Rets.success(Maps.newHashMap("list",list,"map",map));
    }

    /**
     * 导出参数列表
     * @param cfgName
     * @param cfgValue
     * @return
     */
    @RequestMapping(value = "/export",method = RequestMethod.GET)
    @RequiresPermissions(value = {Permission.CFG})
    public Object export(@RequestParam(required = false) String cfgName, @RequestParam(required = false) String cfgValue) {
        Page<Cfg> page = new PageFactory<Cfg>().defaultPage();
        if(StringUtil.isNotEmpty(cfgName)){
            page.addFilter(SearchFilter.build("cfgName", SearchFilter.Operator.LIKE, cfgName));
        }
        if(StringUtil.isNotEmpty(cfgValue)){
            page.addFilter(SearchFilter.build("cfgValue", SearchFilter.Operator.LIKE, cfgValue));
        }
        page.addFilter(SearchFilter.build("delFlag", 0));
        page = cfgService.queryPage(page);
        FileInfo fileInfo = fileService.createExcel("templates/config.xlsx","系统参数.xlsx",Maps.newHashMap("list",page.getRecords()));
        return Rets.success(fileInfo);
    }
    @RequestMapping(method = RequestMethod.POST)
    @BussinessLog(value = "编辑参数", key = "cfgName")
    @RequiresPermissions(value = {Permission.CFG_EDIT})
    public Object save(@ModelAttribute @Valid Cfg cfg){
        if(cfgService.isRepeat(cfg)){
            return Rets.failure("请勿与现有参数的名称重复");
        }
        if(cfg.getId()!=null){
            Cfg old = cfgService.get(cfg.getId());

            LogObjectHolder.me().set(old);
            old.setCfgName(cfg.getCfgName());
            old.setCfgValue(cfg.getCfgValue());
            old.setCfgDesc(cfg.getCfgDesc());
            cfgService.saveOrUpdate(old);
        }else {
            cfgService.saveOrUpdate(cfg);
        }
        return Rets.success();
    }
    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除参数", key = "id")
    @RequiresPermissions(value = {Permission.CFG_DEL})
    public Object remove(@RequestParam Long id){
        logger.info("id:{}",id);
        if (id == null) {
            throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
        }
        Cfg cfg = cfgService.get(id);
        cfg.setDelFlag(1);
        cfgService.update(cfg);
        return Rets.success();
    }

    @RequestMapping(value="saveGroup",method = RequestMethod.POST)
    @BussinessLog(value = "编辑参数")
    @RequiresPermissions(value = {Permission.CFG_EDIT})
    public Object saveGroup(String json){

        Map<String,String> map = Json.fromJson(Map.class,json);
        for(Map.Entry<String,String> entry:map.entrySet()){
            cfgService.update(entry.getKey(),entry.getValue());
        }
        return Rets.success();
    }
}
