package com.emall.flash.api.controller.system;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.entity.shop.Shop;
import com.emall.flash.bean.entity.shop.Tenant;
import com.emall.flash.bean.entity.system.Complaint;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.enumeration.Permission;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.ShopService;
import com.emall.flash.service.shop.TenantService;
import com.emall.flash.service.system.ComplaintService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.DateUtil;
import com.emall.flash.utils.factory.Page;
import com.emall.flash.web.controller.BaseController;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * 投诉管理
 */
@RestController
@RequestMapping("/complaint")
public class ComplaintController extends BaseController {

    @Autowired
    private TenantService tenantService;
    @Autowired
    private ComplaintService complaintService;

    @Autowired
    private ManagerService managerService;

    // 获取数据
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Object list(@RequestParam(required = false) Integer type,
                       @RequestParam(required = false) String contact,
                       @RequestParam(required = false) Integer status,
                       @RequestParam(required = false) String startDate,
                       @RequestParam(required = false) String endDate,
                       @RequestParam(required = false) Long tenantId
                       ) {
        Page<Complaint> page = new PageFactory<Complaint>().defaultPage();
        page.addFilter("contact", SearchFilter.Operator.EQ,contact);
        page.addFilter("type", SearchFilter.Operator.EQ,type);
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null)
            page.addFilter(SearchFilter.build("tenantId",loginUser.getTenantId()));
        else if (tenantId!=null)
            page.addFilter(SearchFilter.build("tenantId",tenantId));
        page.addFilter("status", SearchFilter.Operator.EQ,status);
        page.addFilter("dealTime", SearchFilter.Operator.GTE, DateUtil.parse(startDate,"yyyy-MM-dd HH:mm:ss"));
        page.addFilter("dealTime", SearchFilter.Operator.LTE, DateUtil.parse(endDate,"yyyy-MM-dd HH:mm:ss"));
        page = complaintService.queryPage(page);
        page.getRecords().forEach( f -> {
            if (f.getTenantId() != null) {
                Tenant tenant = tenantService.get(f.getTenantId());
                f.setTenantName(tenant.getName());
            }
        });
        return Rets.success(page);
    }

    // 处理建议
    @RequestMapping(value = "/handleSuggestion" ,method = RequestMethod.PUT)
    public Object handleSuggestion(@RequestBody Complaint complaint) {
        complaint.setDealTime(new Date());
        complaintService.update(complaint);
        return Rets.success();
    }




}
