package com.emall.flash.api.controller.system;

import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.IntegralAmout;
import com.emall.flash.bean.entity.system.MemberBenefits;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.vo.front.Ret;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.cache.CacheDao;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.IntegralService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.service.system.MemberBenefitsService;
import com.emall.flash.utils.RedisCons;
import com.emall.flash.web.controller.BaseController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/memberBenefits")
public class MemberController extends BaseController {

    @Autowired
    private CacheDao cacheDao;

    @Autowired
    private ManagerService managerService;

    @Autowired
    private MemberBenefitsService service;

    @Autowired
    private IntegralService integralService;

    // 查询会员等级
    @RequestMapping(value = "/queryNumber",method = RequestMethod.GET)
    public Object queryNumber(MemberBenefits memberBenefits){
        Long tenantId = null;
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null){
            tenantId = loginUser.getTenantId();
        } else {
            tenantId = memberBenefits.getTenantId();
        }
        int number = service.queryNumber(tenantId);
        return Rets.success(number);
    }
    // 查询列表
    @RequestMapping(value = "/queryMemberBenefits",method = RequestMethod.GET)
    public Object queryMemberBenefits(MemberBenefits memberBenefits){
        Long tenantId = null;
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null){
            tenantId = loginUser.getTenantId();
        } else {
            tenantId = memberBenefits.getTenantId();
        }
        List<MemberBenefits> benefits = service.queryMemberBenefits(memberBenefits,tenantId);
        return Rets.success(benefits);
    }
    // 查询积分列表
    @RequestMapping(value = "/queryAllIntegral",method = RequestMethod.GET)
    public Object queryAllIntegral(IntegralAmout integralAmout) {
        Long tenantId = null;
        User loginUser = managerService.get(JwtUtil.getUserId());
        if (loginUser.getTenantId()!=null){
            tenantId = loginUser.getTenantId();
        } else {
            tenantId = integralAmout.getTenantId();
        }
        List<IntegralAmout> integralAmouts = integralService.findAllIntegral(tenantId);
        return  Rets.success(integralAmouts);
    }

    // 添加会员等级
    @RequestMapping(value = "/add",method = RequestMethod.POST)
    public Object add(@RequestBody MemberBenefits memberBenefits){
        User loginUser = managerService.get(JwtUtil.getUserId());
        memberBenefits.setTenantId(loginUser.getTenantId());
        memberBenefits.setDelFlag(0);
        memberBenefits.setStatus(1);
        if (Boolean.TRUE.equals(service.isRepeat(memberBenefits)))
            return Rets.failure("会员等级名称重复");
        service.insert(memberBenefits);
        cacheDao.set(RedisCons.MEMBER_INFO+":"+ memberBenefits.getLevel(),memberBenefits);
        return  Rets.success(memberBenefits.getId());
    }
    @RequestMapping(value = "/addIntegral",method = RequestMethod.POST)
    public Object addIntegral(@RequestBody IntegralAmout integralAmout) {
        User loginUser = managerService.get(JwtUtil.getUserId());
        integralAmout.setTenantId(loginUser.getTenantId());
        integralAmout.setDelFlag(0);
        integralService.insert(integralAmout);
        return Rets.success(integralAmout.getId());
    }

    // 编辑会员等级
    @RequestMapping(value = "/update",method = RequestMethod.PUT)
    public Object update(@RequestBody MemberBenefits memberBenefits){
        if (Boolean.TRUE.equals(service.isRepeat(memberBenefits)))
            return Rets.failure("会员等级名称重复");
        service.update(memberBenefits);
        cacheDao.set(RedisCons.MEMBER_INFO+":"+ memberBenefits.getLevel(),memberBenefits);
        return Rets.success();
    }
    @RequestMapping(value = "/updateIntegral",method = RequestMethod.PUT)
    public Object updateIntegral(@RequestBody IntegralAmout integralAmout) {
        integralService.update(integralAmout);
        return Rets.success();
    }

    // 删除会员等级
    @RequestMapping(value = "/delete",method = RequestMethod.DELETE)
    public Object delete(MemberBenefits memberBenefitss) {
        MemberBenefits memberBenefits = service.get(memberBenefitss.getId());
        int maxLevel = service.queryMaxNumber();
        if (memberBenefits.getLevel() < maxLevel) {
            return Rets.failure("只能删除等级最大的会员等级");
        } else {
            memberBenefits.setStatus(0);
            memberBenefits.setDelFlag(1);
            service.update(memberBenefits);
            cacheDao.del(RedisCons.MEMBER_INFO+":"+ memberBenefits.getLevel());
            return Rets.success();
        }
    }
    @RequestMapping(value = "/deleteIntegral",method = RequestMethod.DELETE)
    public Object deleteIntegral(IntegralAmout OldIntegralAmout){
        IntegralAmout integralAmout = integralService.get(OldIntegralAmout.getId());
        integralAmout.setDelFlag(1);
        integralService.update(integralAmout);
        return Rets.success();
    }

    @RequestMapping(value = "/getDetail/{id}",method = RequestMethod.GET)
    public Object getDetail(@PathVariable("id") Long id){
        MemberBenefits memberBenefits = service.get(id);
        return Rets.success(memberBenefits);
    }

    @RequestMapping(value = "/getIntegral/{id}",method = RequestMethod.GET)
    public Object getIntegral(@PathVariable("id") Long id){
        IntegralAmout integralAmout = integralService.get(id);
        return Rets.success(integralAmout);
    }
}
