package com.emall.flash.api.controller.system;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.Shop;
import com.emall.flash.bean.entity.system.Model;
import com.emall.flash.bean.entity.system.ModelTenant;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.ModelVo;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.service.system.ModelService;
import com.emall.flash.service.system.ModelTenantService;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.StringUtil;
import com.emall.flash.utils.factory.Page;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@RestController
@RequestMapping("/model")
public class ModelController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private ModelService modelService;
    @Autowired
    private ModelTenantService modelTenantService;
    @Autowired
    private ManagerService managerService;
    /**
     * 获取所有功能键
     */
    @RequestMapping(value = "/list",method = RequestMethod.GET)
    public Object list(@RequestParam(required = false) Integer groups,
                       @RequestParam(required = false) String name) {
        Page<Model> page = new PageFactory<Model>().defaultPage();
        if(StringUtil.isNotEmpty(name)){
            page.addFilter(SearchFilter.build("name", SearchFilter.Operator.LIKE, name));
        }
        if(null != groups){
            page.addFilter(SearchFilter.build("groups", SearchFilter.Operator.EQ, groups));
        }
        page.addFilter( "delFlag",SearchFilter.Operator.EQ,0);
        page.setSort(Sort.by(Sort.Direction.DESC, "createTime"));
        page = modelService.queryPage(page);
        return Rets.success(page);
    }

    @RequestMapping(value = "/add",method = RequestMethod.POST)
    @BussinessLog(value = "添加功能键", key = "id")
//    @RequiresPermissions(value = {Permission.DICT_EDIT})
    public Object add(@RequestBody Model model) {
        model.setDelFlag(0);
        model.setStatus(1);
        if (modelService.isRepeat(model))
            return Rets.failure("功能键名称重复");
        modelService.insert(model);
        return Rets.success(model.getId());
    }

    @RequestMapping(value = "/update",method = RequestMethod.PUT)
    @BussinessLog(value = "修改功能键", key = "id")
//    @RequiresPermissions(value = {Permission.DICT_EDIT})
    public Object update(@RequestBody Model model) {
        if (modelService.isRepeat(model))
            return Rets.failure("功能键名称重复");
        modelService.update(model);
        return Rets.success(model);
    }


    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除功能键", key = "id")
//    @RequiresPermissions(value = {Permission.DICT_DEL})
    public Object delete(@RequestParam Long id) {
        //已分配商圈的不能删
        List<ModelTenant> list = modelTenantService.queryAll(SearchFilter.build("modelId", SearchFilter.Operator.EQ, id));
        if (!list.isEmpty())
            throw new ApplicationException(ApplicationExceptionEnum.DELETE_MODEL_ERROR);
        Model model = modelService.get(id);
        model.setDelFlag(1);
        modelService.update(model);
        return Rets.success();
    }

    @RequestMapping(value = "/addModelTenant",method = RequestMethod.POST)
    @BussinessLog(value = "开通功能键", key = "id")
    public Object addModelTenant(@RequestBody ModelVo modelVo) {
        List<ModelTenant> del;
        List<Long> delList = new ArrayList<>();
        //已经绑定的功能键
        List<ModelTenant> modelTenants = modelTenantService.queryAll(
                SearchFilter.build("tenantId", SearchFilter.Operator.EQ, modelVo.getTenantId()));
        //如果新增功能id为空，则全部删除
        if (modelVo.getModelIds() == null || modelVo.getModelIds().isEmpty()){
            del = new ArrayList<>(modelTenants);
        } else {
            List<Long> oldModelIds = new ArrayList<>();
            if (!modelTenants.isEmpty())
                oldModelIds = modelTenants.stream().map(ModelTenant::getModelId).collect(Collectors.toList());
            // 复制一个原始数组,将作为交集
            List<Long> intersecList = new ArrayList<>(modelVo.getModelIds());
            intersecList.retainAll(oldModelIds);

            // A -C ,即addList
            modelVo.getModelIds().removeAll(intersecList);
            List<Long> addList = new ArrayList<>(modelVo.getModelIds());

            // B -C,即delList
            oldModelIds.removeAll(intersecList);
            delList.addAll(oldModelIds);

            del = modelTenants.stream().filter(l -> delList.contains(l.getTenantId())).collect(Collectors.toList());
            if (!addList.isEmpty()) {
                for (Long modelId : modelVo.getModelIds()) {
                    Model model = modelService.get(modelId);
                    ModelTenant modelTenant = new ModelTenant();
                    modelTenant.setModelId(modelId);
                    modelTenant.setTenantId(modelVo.getTenantId());
                    //默认功能键默认显示
                    if (model.getIsDefault().equals(0))
                        modelTenant.setIsOpen(0);
                    else
                        modelTenant.setIsOpen(1);
                    modelTenant.setSort(model.getSort());
                    modelTenantService.insert(modelTenant);
                }
            }
        }
        if (!del.isEmpty())
            modelTenantService.deleteAll(del);
        return Rets.success(modelVo);
    }

    /**
     * 获取商圈待开通的功能键
     */
    @RequestMapping(value = "/getNoOpenModelsById",method = RequestMethod.GET)
    public Object getNoOpenModelsById(@RequestParam Long id) {
        List<ModelTenant> modelTenants = modelTenantService.queryAll(SearchFilter.build("tenantId", SearchFilter.Operator.EQ, id));
        if (!modelTenants.isEmpty()) {
            List<Long> list = modelTenants.stream().map(ModelTenant::getModelId).collect(Collectors.toList());
            List<Map> allMap = modelService.selectAll(list);
            Map<String,List<Map>> map = new HashMap<>();
            List<Map> homeList = allMap.stream().filter(l -> l.get("groups").equals(1)).collect(Collectors.toList());
            map.put("home",homeList);
            List<Map> myList = allMap.stream().filter(l -> l.get("groups").equals(2)).collect(Collectors.toList());
            map.put("my",myList);
            List<Map> moreList = allMap.stream().filter(l -> l.get("groups").equals(3)).collect(Collectors.toList());
            map.put("more",moreList);
            return Rets.success(map);
        } else {
            List<SearchFilter> filters = Lists.newArrayList();
            filters.add(SearchFilter.build("delFlag",0));
            filters.add(SearchFilter.build("status",1));
            List<Model> all = modelService.queryAll(filters,Sort.by(Sort.Direction.ASC, "isDefault","sort"));
            Map<String,List<Model>> map = new HashMap<>();
            List<Model> homeList = all.stream().filter(l -> l.getGroups().equals(1)).collect(Collectors.toList());
            map.put("home",homeList);
            List<Model> myList = all.stream().filter(l -> l.getGroups().equals(2)).collect(Collectors.toList());
            map.put("my",myList);
            List<Model> moreList = all.stream().filter(l -> l.getGroups().equals(3)).collect(Collectors.toList());
            map.put("more",moreList);
            return Rets.success(map);
        }
    }


    /**
     * 获取商圈开通的功能键
     */
    @RequestMapping(value = "/getModelsById",method = RequestMethod.GET)
    public Object getModelsById(@RequestParam(required = false) Integer isOpen) {
        User user = managerService.get(JwtUtil.getUserId());
        Long tenantId = user.getTenantId();
            List<Map> allMap = modelService.getModelsById(isOpen,tenantId);
            Map<String,List<Map>> map = new HashMap<>();
            List<Map> homeList = allMap.stream().filter(l -> l.get("groups").equals(1)).collect(Collectors.toList());
            map.put("home",homeList);
            List<Map> myList = allMap.stream().filter(l -> l.get("groups").equals(2)).collect(Collectors.toList());
            map.put("my",myList);
            List<Map> moreList = allMap.stream().filter(l -> l.get("groups").equals(3)).collect(Collectors.toList());
            map.put("more",moreList);
            return Rets.success(map);
    }

    @RequestMapping(value = "/setModelTenant",method = RequestMethod.POST)
    @BussinessLog(value = "商圈配置功能键", key = "id")
    public Object setModelTenant(@RequestBody Model model) {
        modelTenantService.update(model.getModelTenants());
        return Rets.success(model);
    }


    @RequestMapping(value = "/updateStatus",method = RequestMethod.PUT)
    @BussinessLog(value = "停启用功能键", key = "id")
    public Object updateStatus(@RequestParam Long id,@RequestParam Integer status) {
        Model model = modelService.get(id);
        model.setStatus(status);
        modelService.update(model);
        return Rets.success();
    }
}
