package com.emall.flash.api.controller.system;

import com.emall.flash.bean.constant.Const;
import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.Tenant;
import com.emall.flash.bean.entity.system.Dept;
import com.emall.flash.bean.entity.system.Role;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.enumeration.Permission;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.node.Node;
import com.emall.flash.bean.vo.node.ZTreeNode;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.TenantService;
import com.emall.flash.service.system.DeptService;
import com.emall.flash.service.system.LogObjectHolder;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.service.system.RoleService;
import com.emall.flash.service.system.impl.ConstantFactory;
import com.emall.flash.utils.BeanUtil;
import com.emall.flash.utils.Convert;
import com.emall.flash.utils.Maps;
import com.emall.flash.utils.StringUtil;
import com.emall.flash.utils.factory.Page;
import com.emall.flash.warpper.RoleWarpper;
import com.emall.flash.web.controller.BaseController;
import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@RestController
@RequestMapping("/role")
public class RoleController extends BaseController {
    private Logger logger = LoggerFactory.getLogger(RoleController.class);

    @Autowired
    private RoleService roleService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private ManagerService managerService;
    @Value("#{'${init.roleId}'.trim().split(',')}")
    private List<Long> initRoleIds;
    @Value("#{'${init.menuId}'.trim().split(',')}")
    private List<String> initMenuIds;
    @Autowired
    private TenantService tenantService;

    @RequestMapping(value = "/list",method = RequestMethod.GET)
    @RequiresPermissions(value = {Permission.ROLE})
    public Object list( @RequestParam(required = false)String name,
                        @RequestParam(required = false) List<Long> tenantIds){
        Page<Role> page = new PageFactory().defaultPage();
        if(StringUtil.isNotEmpty(name)){
            page.addFilter(SearchFilter.build("name", SearchFilter.Operator.LIKE, name));
        }
        User user = managerService.get(JwtUtil.getUserId());
        if (user.getTenantId()!=null)
            page.addFilter(SearchFilter.build("tenantId", SearchFilter.Operator.EQ, user.getTenantId()));
        else if (tenantIds!=null && !tenantIds.isEmpty())
            page.addFilter(SearchFilter.build("tenantId", SearchFilter.Operator.IN, tenantIds));
        page = roleService.queryPage(page);
        for (Role record : page.getRecords()) {
            if (record.getTenantId()!=null) {
                Tenant tenant = tenantService.get(record.getTenantId());
                record.setTenantName(tenant.getName());
                if (record.getPid()!=null) {
                    Role role = roleService.get(record.getPid());
                    if (role != null)
                        record.setPidName(role.getName());
                }
                if (record.getDeptid()!=null) {
                    Dept dept = deptService.get(record.getDeptid());
                    if (dept != null)
                        record.setDeptName(dept.getFullname());
                }
            }
        }
        return Rets.success(page);
    }

    @RequestMapping(method = RequestMethod.POST)
    @BussinessLog(value = "编辑角色", key = "name")
    @RequiresPermissions(value = {Permission.ROLE_EDIT})
    public Object save(@Valid Role role){
        User user = managerService.get(JwtUtil.getUserId());
        if(role.getId()==null) {
            role.setTenantId(user.getTenantId());
            roleService.insert(role);
        }else{
            //商圈管理员不能更改商圈管理员和超级管理员角色
            if (user.getUserType().equals(1) && (initRoleIds.contains(role.getId()))) {
                return Rets.failure("不能修改当前角色");
            }
            Role old = roleService.get(role.getId());
            LogObjectHolder.me().set(old);
            roleService.update(role);
        }
        return Rets.success();
    }
    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除角色", key = "roleId")
    @RequiresPermissions(value = {Permission.ROLE_DEL})
    public Object remove(@RequestParam Long roleId){
        logger.info("id:{}",roleId);
        //商圈管理员不能更改商圈管理员和超级管理员角色
        User user = managerService.get(JwtUtil.getUserId());
        if (user.getUserType().equals(1) && (initRoleIds.contains(roleId))) {
            return Rets.failure("不能删除初始角色");
        }
        if (roleId==null) {
            throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
        }
        if(roleId.intValue()<2){
            return Rets.failure("不能删除初始角色");
        }
        List<User> userList = managerService.queryAll(SearchFilter.build("roleid", SearchFilter.Operator.EQ,String.valueOf(roleId)));
        if(!userList.isEmpty()){
            return Rets.failure("有用户使用该角色，禁止删除");
        }
        //不能删除超级管理员角色
        if(roleId.intValue() ==Const.ADMIN_ROLE_ID){
            return Rets.failure("禁止删除超级管理员角色");
        }
        //缓存被删除的角色名称
        LogObjectHolder.me().set(ConstantFactory.me().getSingleRoleName(roleId));
        roleService.delRoleById(roleId);
        return Rets.success();
    }

    @RequestMapping(value = "/savePermisson",method = RequestMethod.POST)
    @BussinessLog(value = "配置角色权限", key = "roleId")
    @RequiresPermissions(value = {Permission.ROLE_EDIT})
    public Object setAuthority(Long roleId, String
            permissions) {
        if (roleId == null) {
            throw new ApplicationException(ApplicationExceptionEnum.REQUEST_NULL);
        }
        //商圈管理员不能更改商圈管理员和超级管理员角色
        User user = managerService.get(JwtUtil.getUserId());
        if (user.getUserType().equals(1) && (initRoleIds.contains(roleId))) {
            return Rets.failure("不能修改当前角色权限");
        }
        //商圈管理员不能分配非商圈的权限
        if (user.getUserType().equals(1)) {
            String[] permissionList = permissions.split(",");
            List<String> list = new ArrayList<>(Arrays.asList(permissionList));
            list.retainAll(initMenuIds);
            if (!list.isEmpty())
                return Rets.failure("不能分配非商圈权限");
        }
        roleService.setAuthority(roleId, permissions);
        return Rets.success();
    }


    /**
     * 获取角色树
     */
    @RequestMapping(value = "/roleTreeListByIdUser", method = RequestMethod.GET)
    @RequiresPermissions(value = {Permission.ROLE})
    public Object roleTreeListByIdUser(Long idUser) {
        User loginUser = managerService.get(JwtUtil.getUserId());
        User user = managerService.get(idUser);
        String roleIds = user.getRoleid();
        List<ZTreeNode> roleTreeList = null;
        if (StringUtil.isEmpty(roleIds)) {
            roleTreeList = roleService.roleTreeList(loginUser.getTenantId());
        } else {
            Long[] roleArray = Convert.toLongArray(",", roleIds);
            roleTreeList = roleService.roleTreeListByRoleId(roleArray,loginUser.getTenantId());

        }
        List<Node> list = roleService.generateRoleTree(roleTreeList);
        List<Long> checkedIds = Lists.newArrayList();
        for (ZTreeNode zTreeNode : roleTreeList) {
            if (zTreeNode.getChecked() != null && zTreeNode.getChecked()) {
                checkedIds.add(zTreeNode.getId());
            }
        }
        return Rets.success(Maps.newHashMap("treeData", list, "checkedIds", checkedIds));
    }
}
