package com.emall.flash.api.service;

import com.emall.flash.bean.entity.shop.Order;
import com.emall.flash.bean.entity.shop.ShopUser;
import com.emall.flash.bean.enumeration.shop.OrderEnum;
import com.emall.flash.service.shop.OrderService;
import com.emall.flash.utils.HttpUtil;
import com.github.binarywang.wxpay.bean.notify.WxPayNotifyResponse;
import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyResult;
import com.github.binarywang.wxpay.bean.notify.WxPayRefundNotifyResult;
import com.github.binarywang.wxpay.bean.order.WxPayMpOrderResult;
import com.github.binarywang.wxpay.bean.request.WxPayRefundRequest;
import com.github.binarywang.wxpay.bean.request.WxPayUnifiedOrderRequest;
import com.github.binarywang.wxpay.bean.result.WxPayRefundResult;
import com.github.binarywang.wxpay.constant.WxPayConstants;
import com.github.binarywang.wxpay.exception.WxPayException;
import com.github.binarywang.wxpay.service.WxPayService;
import org.apache.commons.io.IOUtils;
import org.nutz.json.Json;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 微信支付
 * @author ：enilu
 * @date ：Created in 2020/3/17 20:26
 */
@Service
public class WeixinPayService {
    @Autowired
    private WxPayService wxPayService;
    @Autowired
    private OrderService orderService;
    @Value("${wx.refund-notify-url}")
    private String refundNotifyUrl;
    private Logger logger = LoggerFactory.getLogger(WeixinPayService.class);


    /**
     * 退款
     *
     * @return
     */
    public WxPayRefundResult refund(Order order) {
        WxPayRefundRequest wxPayRefundRequest = new WxPayRefundRequest();
        wxPayRefundRequest.setTransactionId(order.getPayId());
        wxPayRefundRequest.setOutRefundNo(String.valueOf(System.currentTimeMillis()));
        BigDecimal totalPrice = order.getTotalPrice();
        wxPayRefundRequest.setTotalFee(totalPrice.intValue());
        wxPayRefundRequest.setRefundFee(totalPrice.intValue());
        wxPayRefundRequest.setNotifyUrl(refundNotifyUrl);
        try {
            WxPayRefundResult refund = wxPayService.refund(wxPayRefundRequest);
            if (refund.getReturnCode().equals("SUCCESS") && refund.getResultCode().equals("SUCCESS")) {
                order.setStatus(OrderEnum.OrderStatusEnum.REFUND_ING.getId());
                order.setRefundTime(new Date());
                orderService.updateOrder(order);
                return refund;
            }

        } catch (WxPayException e) {
            logger.error("微信退款异常",e);
        }
        return null;
    }


}
